function [C, invC, ldC, nmodeK, nmodeN, dC_dtheta, dC_dU, dC_dgamma] = ... 
    computeNModeKernel(Us, Hps, idx, theta, beta, Kn, invKn, ldKn, nmodeKn, nmodeNn)
% Compute the kernel matrix for a multifactor GP
%
% Us - cell array of input matrices (Nxd) for each factor
% Hps - cell array of hyperparameters for each factor, this determines 
%       whether linear (leave empty) or RBF kernel (length scale) is used.
% idx - N x number of factors matrix, used to indicate which factors are 
%       shared across which data vectors
%       e.g., [1 2 1; 
%              1 1 1; 
%              2 2 2] means data 1 and 2 share factors 1 and 3, while data
%              1 and 3 share factor 2
% theta - overall scale of the kernel
% beta - observation noise
%
% C, invC - N x N kernel matrix and its inverse
% dC_dtheta, dC_dU, dC_dgamma - derivatives of the kernel w.r.t. inputs and
% hyperparameters

numData = size(idx,1);
numModes = size(idx,2);
if (nargin > 5)
    N = size(Kn,1); 
    M = numData - N; 
    Ka = ones(N, M);
    Kb = ones(M, M);
    nmodeK = cell(numModes, 1);
    for n = 1:numModes
        U = Us{n}(1:N,:);
        V = Us{n}(N+1:end,:);
        gamma = Hps{n};

        if isempty(gamma)
            Kna = U*V';
            Knb = V*V';
        else
            N2 = dist2(U, V);
            Kna = exp(-N2*gamma/2);
            N2_2 = dist2(V, V);
            Knb = exp(-N2_2*gamma/2);
            nmodeN{n} = [nmodeNn{n} N2; N2' N2_2]; 
        end
        nmodeKa{n} = Kna(idx(1:N,n), idx(N+1:end,n)-max(idx(1:N,n)));
        Ka = Ka.*nmodeKa{n};
        nmodeKb{n} = Knb(idx(N+1:end,n)-max(idx(1:N,n)), idx(N+1:end,n)-max(idx(1:N,n)));
        Kb = Kb.*nmodeKb{n};
        nmodeK{n} = [nmodeKn{n} nmodeKa{n}; nmodeKa{n}' nmodeKb{n}]; 
    end
    A = theta*Ka; 
    B = theta*Kb + eye(M)*(1/beta);
    C = [Kn A; A' B]; 
    K = (C - eye(numData)*(1/beta))*(1/theta); 
else
    K = ones(numData, numData);
    nmodeK = cell(numModes, 1);
    for n = 1:numModes
        U = Us{n};
        gamma = Hps{n};
        if isempty(gamma)
            Kn = U*U';
        else
            N2 = dist2(U, U);
            Kn = exp(-N2*gamma/2);
            nmodeN{n} = N2;
        end

        nmodeK{n} = Kn(idx(:,n), idx(:,n));

        K = K.*nmodeK{n};
    end

    C = theta*K + eye(numData)*1/beta;
end

if nargout > 1
    if (nargin > 5)
        tmp = invKn*A; 
        D = B - A'*tmp;
        Dinv = pdinv(D);
        AA = invKn + tmp*Dinv*tmp';
        BB = -tmp*Dinv;
        CC = Dinv;

        invC = [AA, BB; BB' CC];
        ldC = ldKn + logdet(D); 
    else
        invC = pdinv(C);
        ldC = logdet(C);
    end
end

if nargout > 4
    dC_dtheta = theta*K/theta; 
    dC_dU = cell(numModes,1);
    for n = 1:numModes
        U = Us{n};
        gamma = Hps{n};
        dC_dU{n} = cell(size(U,2),1);
        if isempty(gamma)
            for q = 1:size(U,2)
                K3 = repmat(U(idx(:,n), q)', numData, 1);
                tmpK = ones(numData, numData); 
                for j = 1:numModes
                    if (j ~= n)
                        tmpK = tmpK.*nmodeK{j}; 
                    end
                end
                %dC_dU{n}{q} = (tril(K3) + triu(K3)).*(theta*K./nmodeK{n});
                dC_dU{n}{q} = (tril(K3) + triu(K3)).*(theta*tmpK);
            end
            dC_dgamma{n} = [];
        else
            for q = 1:size(U,2)
                K1 = repmat(U(idx(:,n), q), 1, numData);
                K2 = repmat(U(idx(:,n), q)', numData, 1);
                dC_dU{n}{q} = -theta*gamma*(K1-K2).*K;
            end
            dC_dgamma{n} = -0.5*theta*nmodeN{n}(idx(:,n), idx(:,n)).*K; 
        end
    end
end
